@@ -181,10 +181,18 @@ class Track(ScrobblableMixin):
     def get_absolute_url(self):
         return reverse('music:track_detail', kwargs={'slug': self.uuid})
 
+    @property
+    def subtitle(self):
+        return self.artist
+
     @property
     def mb_link(self):
         return f"https://musicbrainz.org/recording/{self.musicbrainz_id}"
 
+    @property
+    def info_link(self):
+        return self.mb_link
+
     @classmethod
     def find_or_create(
         cls, artist_dict: Dict, album_dict: Dict, track_dict: Dict
